#!/bin/bash

# Abort execution when an error occurs
set -e

# Set working directory properly
cd "$(dirname $0)"
WORKDIR="$(pwd)"
COMPOSE_PROJECT_NAME="fragjetzt"

function main {

  # Read Input
  while true; do
    case "$1" in

      # GENERAL SECTION
      --recreate-env )
        RECREATE_ENV=true; shift ;;
      --deepl-key )
        [ -z ${2} ] && warn "No deepl key given."
        DEEPL_KEY="${2}"; shift 2 ;;

      # DISABLE SERVICES SECTION
      --no-backend )
        NO_BACKEND=true; shift ;;
      --no-frontend )
        NO_FRONTEND=true; shift ;;
      --no-ws-gateway )
        NO_WS_GATEWAY=true; shift ;;
      --no-email )
        NO_EMAIL=true; shift ;;
      --no-psql )
        NO_PSQL=true; shift ;;
      --no-rabbit )
        NO_RABBIT=true; shift ;;

      # PRODUCTION SECTION
      --prod )
        PROD=true; shift ;;
      --domain )
        [ -z ${2} ] && warn "No domain given."
        DOMAIN="${2}"; shift 2 ;;
      --ssl-priv-key )
        [ -z ${2} ] && warn "No priv key location given."
        SSL_PRIV_KEY="${2}"; shift 2 ;;
      --ssl-pub-key )
        [ -z ${2} ] && warn "No pub key location given."
        SSL_PUB_KEY="${2}"; shift 2 ;;

      # CLOSING SECTION
      "" ) break ;;
      * ) warn "unknown option: $1"; exit 1 ;;

    esac
  done

  if [ ! -z ${PROD} ] && [ ! $(command -v openssl) ]; then
     warn "Production environment needs openssl. Abort."
     exit 1
  fi

  # Check if input is set properly for production
  if [ ! -z ${PROD} ] && (
    [ -z ${DOMAIN} ] || [ -z ${SSL_PRIV_KEY} ] || [ -z ${SSL_PUB_KEY} ]
  ); then
    warn "On or more arguments are missing for production setup."
    warn "This may be intentional, but then the setup must be adjusted."
    warn "Reqired:"
    warn " --domain       (the fully qualified domain name)"
    warn " --ssl-priv-key (the location of the private ssl key)"
    warn " --ssl-pub-key  (the location of the public ssl key)"
  elif [ ! -z ${PROD} ]; then
    ok "Using production configuration with following values:"
    ok " - Domain:                ${DOMAIN}"
    ok " - SSL priv key location: ${SSL_PRIV_KEY}"
    ok " - SSL pub key location:  ${SSL_PUB_KEY}"
  fi

  # Create mandatory directories
  mkdir -p secrets logs/nginx

  # generate environment variables
  generateEnvironments

  # generate secret variables
  generateSecrets

  # write out environment files
  createEnvironments

  # copy configurations
  createConfigs

  # write out docker compose override file
  createComposeOverride
}

# Only generates the environment variables to use later
function generateEnvironments {
  if [ -z ${PROD} ]; then
    # The DEV variables
    DOMAIN="localhost"
    LOGGING_LEVEL_ROOT="INFO"
    APP_MAIL_HOST="fragjetzt-mailhog"
    SERVER_ROOT_URL="http:\/\/${DOMAIN}:4200"
  else
    # The PROD variables
    LOGGING_LEVEL_ROOT="ERROR"
    APP_MAIL_HOST="fragjetzt-postfix"
    SERVER_ROOT_URL="https:\/\/${DOMAIN}"
  fi

  APP_MAIL_SENDER_ADDRESS="postmaster@${DOMAIN}"
  MAIL_DOMAIN="${DOMAIN}"
  MAIL_HOST="${DOMAIN}"
  ALLOWEDORIGINS="${SERVER_ROOT_URL}"
}

# Only generates the environment variables to use later
function generateSecrets {
  if [ -z ${PROD} ]; then
    # The DEV variables
    POSTGRES_SECRET="fragjetzt"
    RABBITMQ_SECRET="guest"
    JWT_SECRET=$(openssl rand --hex 32)
  else
    # The PROD variables
    POSTGRES_SECRET=$(openssl rand --hex 32)
    RABBITMQ_SECRET=$(openssl rand --hex 32)
    JWT_SECRET=$(openssl rand --hex 32)
  fi
}

# Write out the environment files
function createEnvironments {
  # Check if env files are present and should be overwritten
  if [ -f environments/generated.env ] &&  [ -z ${RECREATE_ENV} ]; then
    info "Environment is already set and will not be overwritten."
    info "  use \"--recreate-env\" if you would like to override"
    return
  fi

  # Copy all template files to env files
  for FILE in environments/*.template; do
    cp "${FILE}" "${FILE%.template}.env"
  done

  # Set Environment Strings to be substituted
  ENV_STRINGS="
    APP_MAIL_HOST=${APP_MAIL_HOST}
    APP_MAIL_SENDER_ADDRESS=${APP_MAIL_SENDER_ADDRESS}
    SERVER_ROOT_URL=${SERVER_ROOT_URL}
    LOGGING_LEVEL_ROOT=${LOGGING_LEVEL_ROOT}
    MAIL_DOMAIN=${MAIL_DOMAIN}
    MAIL_HOST=${MAIL_HOST}
    ALLOWEDORIGINS=${ALLOWEDORIGINS}
    SPRING_RABBITMQ_PASSWORD=${RABBITMQ_SECRET}
    RABBITMQ_DEFAULT_PASS=${RABBITMQ_SECRET}
    STOMP_RELAY_PASSWORD=${RABBITMQ_SECRET}
    RABBITMQ_PASSWORD=${RABBITMQ_SECRET}
    SPRING_R2DBC_PASSWORD=${POSTGRES_SECRET}
    POSTGRES_PASSWORD=${POSTGRES_SECRET}
    SPRING_JWT_SECRET=${JWT_SECRET}
    SECURITY_JWT_SECRET=${JWT_SECRET}
  "

  # Run the substitution
  for FILE in environments/*.env; do
    for ENV_STRING in ${ENV_STRINGS}; do
      sed -i "s/^${ENV_STRING%%=*}=\$/${ENV_STRING%%=*}=${ENV_STRING#*=}/" "${FILE}"
    done
  done

  # Save the datetime of environment creation
  date > environments/generated.env
}

function createConfigs {
  echo "COMPOSE_PROJECT_NAME=${COMPOSE_PROJECT_NAME}" > .env

  if [ -z ${PROD} ]; then
    cp configs/rabbitmq_plugins.dev configs/rabbitmq_plugins
    cp configs/nginx.dev configs/nginx.conf
    cp configs/postfix.dev configs/postfix
  else
    cp configs/rabbitmq_plugins.prod configs/rabbitmq_plugins
    cp configs/nginx.prod configs/nginx.conf
    cp configs/postfix.prod configs/postfix

    sed -i "s/:FQDN/${DOMAIN}/g" configs/nginx.conf
    if [ ! -f secrets/dhparam-4096.pem ]; then
      info "Creating dhparams for SSL Setup. This may take a while."
      openssl dhparam -out secrets/dhparam-4096.pem 4096
    fi

    echo "PRIV_SSL_KEY_LOCATION=${SSL_PRIV_KEY}" >> .env
    echo "PUB_SSL_KEY_LOCATION=${SSL_PUB_KEY}" >> .env
  fi

  if [ ! -z ${DEEPL_KEY} ]; then
    sed -i "s/:DEEPL_AUTH_KEY/${DEEPL_KEY}/g" configs/nginx.conf
  else
    [ -z ${NO_FRONTEND} ] && warn "No Deepl Auth Key given, Deepl AI will not work."
  fi
}

function createComposeOverride {
  cat override-modules/head > docker-compose.override.yml

  if [ -z ${PROD} ]; then
    [ -z ${NO_BACKEND} ]    && override backend.dev     || disable backend
    [ -z ${NO_FRONTEND} ]   && override frontend.dev    || disable frontend
    [ -z ${NO_WS_GATEWAY} ] && override ws-gateway.dev  || disable ws-gateway
    [ -z ${NO_EMAIL} ]      && override email.dev       || disable email
    [ -z ${NO_PSQL} ]       && override postgres.dev    || disable postgres
    [ -z ${NO_RABBIT} ]     && override rabbitmq.dev    || disable rabbitmq
    override objects.dev
  else
    [ -z ${NO_BACKEND} ]                                || disable backend
    [ -z ${NO_FRONTEND} ]   && override frontend.prod   || disable frontend
    [ -z ${NO_WS_GATEWAY} ]                             || disable ws-gateway
    [ -z ${NO_EMAIL} ]                                  || disable email
    [ -z ${NO_PSQL} ]                                   || disable postgres
    [ -z ${NO_RABBIT} ]                                 || disable rabbitmq
    override objects.prod
  fi

}

function disable {
  sed "s/:SERVICE/${1}/g" override-modules/service.disabled >> docker-compose.override.yml
}

function override {
  cat "override-modules/${1}" >> docker-compose.override.yml
}

function warn {
  echo -e "\e[1;31mWARN: ${@}\e[0m"
}

function ok {
  echo -e "\e[1;32mINFO: ${@}\e[0m"
}

function info {
  echo -e "\e[1;33mINFO: ${@}\e[0m"
}

main $@
